<?php
namespace TecDiet\Controllers;

use Doctrine\ORM\EntityManager;
use \Zeedhi\Framework\DependencyInjection\InstanceManager;

class Integration extends \Zeedhi\Framework\Controller\Simple {
    
    protected $connection;
    protected $integrationService;
    protected $logPath;
    protected $pendingPath;
    
    public function __construct(EntityManager $entityManager, $integrationService, InstanceManager $instanceManager) {
	    $this->connection = $entityManager->getConnection();
	    $this->integrationService = $integrationService;
	    $clientFolder = $this->connection->getParams();
        $clientFolder = (isset($clientFolder['user'])) ? $clientFolder['user'] : "undefined";
        $this->logPath = $instanceManager->getParameter('integrationLogPath') . "/$clientFolder";
        $this->pendingPath = $this->logPath . "/pending";
    }
	    
	/*{
        "prontuario": 10115483,
        "nome": "ADELSON AZEVEDO DA SILVA",
        "sexo": "M",
        "data_nascimento": "1955-04-16 00:00:00",
        "unidade": "UNIDADE CLÍNICA",
        "andar": "15",
        "leito": "1509I",
        "codigo_internacao": 51786,
        "data_internacao": "2024-03-22 23:39:00",
        "data_alta": null,
        "data_inicio": "2024-06-10 12:00:00",
        "data_criacao": "2024-06-10 08:48:00.568",
        "codigo_dieta": 240,
        "descricao_dieta": "PARENTERAL",
        "observacao": "NPT - 1 COMPLETA 73ml/h\r\n\r\nJEJUNOSTOMIA: 35ML/H (VIA SONDA DE PEZZER EM ABDOME)\r\n\r\nSNE ABERTA !!!!"
    }
    */
    
    private function callApi() {
        try {
        $curl = curl_init();
            curl_setopt_array($curl, array(
              CURLOPT_URL => 'https://apinutri.hc.ufg.br/api/paciente/fetch',
              CURLOPT_RETURNTRANSFER => true,
              CURLOPT_ENCODING => '',
              CURLOPT_MAXREDIRS => 10,
              CURLOPT_TIMEOUT => 0,
              CURLOPT_FOLLOWLOCATION => true,
              CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
              CURLOPT_CUSTOMREQUEST => 'GET',
              CURLOPT_POSTFIELDS =>'{
                "data_internacao": "2024-06-10"
              }',
              CURLOPT_HTTPHEADER => array(
                'Authorization: Bearer eyJ0eXAiOiJKV1QiLCJhbGciOiJIUzI1NiJ9.eyJ0b2tlbiI6IiQyeSQxMCRSSGowNk9VTDF2XC91anN2bHBvTFkyZXpXQWdlYUhtQnNiWXJoTjRMY3RReVdwMGRQeVI1aEsifQ.Jm8cMTmWPCF1hr0vID7r5uRpNxEoLgcOcphQapCNKVo',
                'Content-Type: application/json'
              ),
            ));
            
            $response = curl_exec($curl);
            $httpCode = curl_getinfo($curl, CURLINFO_HTTP_CODE);
            $err    = curl_errno($curl);
            $errmsg = curl_error($curl);
            
            curl_close($curl);
            
            if($err || empty($response) || $httpCode !== 200){
                echo "ERRO NA REQUISIÇÃO: ". $errmsg. " \n";
                die;
            }
            
            $resultApi = json_decode($response, true);
            if ($resultApi['error'] !== false || $resultApi['success'] !== true) {
                echo "ERRO NO RETORNO DA API: ". $errmsg. " \n";
                die;
            }
            return $resultApi;
        } catch (\Exception $e) {
            $this->connection->rollback();
            echo "ERRO NA REQUISIÇÃO PARA API EXTERNA: ".$e->getMessage()."\n";
            die;
        }
    }
    
    public function createFiles() {
        $resultApi = $this->callApi();
        try {
            $patientsToProccess = $resultApi['data'];
            
            foreach($patientsToProccess as $patient) {
                $folder = $this->pendingPath  . "/" . date("Y") . "/" . date("m") . "/" . date("d");
                $fileName = $patient["prontuario"]."-".round(microtime(true) * 1000).".json";
                $currentFile = $folder . "/" . $fileName;
                
                if(!file_exists($folder)){
                    mkdir($folder, 0775, true);
                }
                $content = json_encode($patient, JSON_PRETTY_PRINT | JSON_UNESCAPED_UNICODE);
                file_put_contents($currentFile, $content);
                
            }
        } catch (\Exception $e) {
            $this->connection->rollback();
            echo "ERRO NA GRAVAÇÃO DE ARQUIVOS: ".$e->getMessage()."\n";
            die;
        }
    }

	public function integrate() {
	   // $this->integrationService->connectIntegration();
	    $this->connection->beginTransaction();
        
        // $resultApi = $this->callApi();
        // $patientsToProccess = $resultApi['data'];
        
        try {
            $filial = $this->integrationService->getFilial();
	    } catch (\Exception $e) {
            echo "ERRO NA REQUISIÇÃO PARA API EXTERNA: ".$e->getMessage()."\n";
            die;
        }
        
        foreach ($patientsToProccess as $patient) {
            $this->connection->beginTransaction();
            try {
                $patient['CDFILIAL'] = $filial['CDFILIAL'];
                /*VERIFICAR SE JÁ EXISTE O PACIENTE CADASTRADO*/
                if (!$this->patientExists($patient)) $this->insertPatient($patient);
                
                /*VERIFICAR SE JÁ EXISTE O ANDAR CADASTRADO*/
                if (!$this->storeyExists($patient)) $this->integrationService->insertStorey($patient);
                
                // não sendo enviado ALA
                
                break;
                // VERIFICA SE JÁ EXISTE O ALA CADASTRADA
                if (!$this->hallExists($patient)) $this->integrationService->insertHall($patient);
                
                // VERIFICA SE EXISTE CADASTRADA A LIGAÇÃO DE ALA E ANDAR
                if (!$this->hallStoreyExists($patient)) $this->integrationService->insertStoreyHall($patient);
                
                // VERIFICA SE JÁ EXISTE O LEITO CADASTRADO
                if (!$this->bedExists($patient)) $this->integrationService->insertBed($patient);
                
                // VERIFICA SE JÁ EXISTE A EQUIPE MÉDICA CADASTRADA
                if (!$this->medTeamExists($patient)) $this->integrationService->insertMedTeam($patient);
                
                // VERIFICA SE JÁ EXISTE O PADRAO DE INTERNACAO CADASTRADO
                if (!$this->padInterExists($patient)) $this->integrationService->insertPadInter($patient);
                
                
                /*VERIFICAR A TROCA DE LEITO*/
                $inter = $this->integrationService->getInter($patient);
                if (empty($inter)) {
                    $this->integrationService->insertInter($patient);
                } else if ($inter['CDLEITO'] !== $patient['leito'] || $patient['data_alta'] !== null) {
                    $this->integrationService->updateInter($patient);
                    if ($inter['CDLEITO'] !== $patient['leito']) {
                        /*DESOCUPA O LEITO ANTERIOR*/
                        $this->integrationService->updateBed($patient, 'N');
                        
                        /*GERAR NOVA PRESCRIÇÃO PARA O NOVO LEITO*/
                        $presDiet = $this->integrationService->getPresDiet($patient);
                        
                        $patient['P_DTPRESDIET'] = $presDiet['DTPRESDIET'];
                        $this->integrationService->insertPresDiet($patient);
                        
                        /*INCLUIR AS OBSERVAÇÕES - LAUDO*/
                        $laudPresc = $this->integrationService->getLaudPresc($patient);
                        if (\Zeedhi\Framework\Util\Functions::arrayKeyExists("IDDIETAESP", $laudPresc) && 
                            \Zeedhi\Framework\Util\Functions::arrayKeyExists("DSLAUDPRES", $laudPresc) && 
                            $laudPresc['IDDIETAESP'] !== null && $laudPresc['DSLAUDPRES'] !== null) {
                            $patient['IDDIETAESP'] = $laudPresc['IDDIETAESP'];
                            $patient['DSLAUDPRES'] = $laudPresc['DSLAUDPRES'];
                            $patient['CDTIPOLAUDO'] = $laudPresc['CDTIPOLAUDO'];
                            $this->integrationService->insertLaudPresc($patient);
                        }
                        
                        /*OBSERVAÇÃO*/
                        $observation = $this->integrationService->getObservation($patient);
                        if (\Zeedhi\Framework\Util\Functions::arrayKeyExists("IDOBSPRESC", $laudPresc) && 
                            \Zeedhi\Framework\Util\Functions::arrayKeyExists("DSOBSPRESC", $laudPresc) && 
                            $laudPresc['IDOBSPRESC'] !== null && $laudPresc['DSOBSPRESC'] !== null) {
                            $patient['IDOBSPRESC'] = $laudPresc['IDOBSPRESC'];
                            $patient['DSOBSPRESC'] = $laudPresc['DSOBSPRESC'];
                            $this->integrationService->insertObservation($patient);
                        }
                        
                        $ocorrPresc = $this->integrationService->getOcorrPresc($patient);
                        foreach($ocorrPresc as $ocorr) {
                            $this->integrationService->insertOcorrPresc(array_merge($patient, $ocorr));
                        }
                    }
                }
                
                if ($params['data_alta'] !== null) {
                    $this->integrationService->updateBed($params, 'N');
                } else {
                    $this->integrationService->updateBed($params, 'S');
                }
                echo "PACIENTE EXECUTADA\n";
                $this->connection->commit();
            } catch (\Exception $e) {
                $this->connection->rollback();
                echo $e->getMessage()."\n";
            }
        }
        echo "INTEGRAÇÃO EXECUTADA\n";
    }
    
    private function patientExists($params) {
        $getPatient = $this->integrationService->getPatient($params);
        return (!empty($getPatient)) ? true : false;
    } 
    
    private function insertPatient($params) {
        $dtNasci = \DateTime::createFromFormat('Y-m-d H:i:s', $params['data_nascimento'], new \DateTimeZone('America/Sao_Paulo'));
        $insert = array(
            'P_CDFILIAL' => '0001',
            'P_CDPACIENTE' => $params['prontuario'],
            'P_NMPACIENTE' => $params['nome'],
            'P_IDSEXOPACI' => $params['sexo'] === "M" ? '0' : '1',
            'P_DTNASCPACI' => $dtNasci->format('d/m/Y H:i:s'),
            'P_IDPACIENT' => '0',
        );
        $this->integrationService->insertPatient($insert);
    }
    
    private function storeyExists($params) {
        $getStorey = $this->integrationService->getStorey($params);
        return (!empty($getStorey)) ? true : false;
    }
    
    private function hallExists($params) {
        $getHall = $this->integrationService->getHall($params);
        return (!empty($getHall)) ? true : false;
    }
    
    private function hallStoreyExists($params) {
        $getStoreyHall = $this->integrationService->getStoreyHall($params);
        return (!empty($getStoreyHall)) ? true : false;
    }
    
    private function bedExists($params) {
        $getBed = $this->integrationService->getBed($params);
        return (!empty($getBed)) ? true : false;
    }
    
    private function medTeamExists($params) {
        $getMedTeam = $this->integrationService->getMedTeam($params);
        return (!empty($getMedTeam)) ? true : false;
    }
    
    private function padInterExists($params) {
        $getPadInter = $this->integrationService->getPadInter($params);
        return (!empty($getPadInter)) ? true : false;
    }
}