<?php
/**
 * Created by PhpStorm.
 * User: francismararaujo
 * Date: 23/10/14
 * Time: 17:07
 */

namespace TecDiet\Controllers;

use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DTO\Response\Message;
use Zeedhi\Framework\DTO\Response\Error;

use TecDiet\Services\UserService;

class UserController extends \Zeedhi\Framework\Controller\Simple {

    protected $userService;

    public function __construct(UserService $userService) {
        $this->userService = $userService;
    }

    public function updateUser(Request\Row $request, Response $response) {

        $row = $request->getRow();

        if($row) {

            try{
                $this->userService->updateUser($row);
            } catch(\Exception $e) {
                $response->setError(new Error($e->getMessage(), $e->getCode()));
            };

        }

    }

    public function save(Request $request, Response $response) {
        $users = $request->getDataSet()->getRows();

        if($users) {
        
            for($i=0;$i<count($users);$i++) {
            
                $user = $users[$i];

                try {
            
                    $this->userService->save($user);

                } catch(\Exception $e) {
                    $response->setError(new Error($e->getMessage(), $e->getCode()));
                };

            }

        }

    }

    public function retornaFiliaisUsuario(Request\Filter $request, Response $response){
        try {
            $filter = $request->getFilterCriteria()->getConditions();
            $operador = $filter[0]['value'];
            $unidades = $this->userService->retornaFiliaisUsuario($operador);
            $response->addDataSet(new DataSet('Units', $unidades));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }

    public function filtraFiliaisUsuario(Request\Row $request, Response $response){
        $row = $request->getRow();
        try {
            $operador = $row['CDOPERADOR'];
            $filiais = $row['CDFILIAIS'];
            $unidades = $this->userService->filtraFiliaisUsuario($operador, $filiais);
            $response->addDataSet(new DataSet('Units', $unidades));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }

    public function retornaUsuarioAtivo(Request\Filter $request, Response $response) {
        try {
            $filter = $request->getFilterCriteria()->getConditions();
            $dsemailoper = $filter[0]['value'];
            $nrorg = $filter[1]['value'];

            $ativo = $this->userService->retornaUsuarioAtivo($dsemailoper, $nrorg);
            $response->addDataSet(new DataSet('usuario', array('ativo' => $ativo)));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }

}