var AppointmentController = function(eventAggregator, localStorageEngine, templateManager, ScreenService, Environment, AppointmentService, EvaluationDataService, Validation, ZHPromise) {

    var containerMain;
    var widgetNavigationStripe;
    var widgetMain;
    var prevValue;

    var MALE = 'M';
    var FEMALE = 'F';

    // LIST

    this.widgetAfterMoveRow = function(dataSource, appointment) {

        Environment.setCurrentAppointment(appointment);
        Environment.setAppointmentDataSource(dataSource);
        Environment.setCurrentAppointmentDate(appointment.date);

        ScreenService.openWindow('Appointment');

    };

    // VIEW

    this.navigationStripeAfterMoveRow = function(appointment) {

        Environment.setCurrentAppointment(appointment);
        Environment.setCurrentAppointmentDate(appointment.date);

        widgetMain.reload().then(function() {
            widgetMain.moveToFirst();
        });

    };

    var _initNavigationStripe = function() {
        widgetNavigationStripe.currentRow = Environment.getCurrentAppointment();
        widgetNavigationStripe.dataSource.data = Environment.getAppointmentDataSource();
    };

    var _initContainer = function(container) {

        containerMain = container;
        widgetNavigationStripe = container.getWidget('WidgetNavigationStripe');
        widgetMain = container.getWidget('WidgetMain');

    };

    this.containerAfterinit = function(container) {

        _initContainer(container);
        _initNavigationStripe();

    };

    // ACTIONS
    this.addEvaluation = function(appointment, evaluation) {

        Environment.setCurrentAppointment(appointment);
        ScreenService.openWindow(evaluation);

    };

    // ANTHROPOMETRIC EVALUATION

    this.openWindow = function(methods) {

        EvaluationDataService.setMethods(methods);
        ScreenService.openWindow('EvaluationData');

    };

    this.closeSwipe = function(widget) {

        ScreenService.goBack();
        widget.activate();

    };

    // EDIT
    this.openSwipe = function(container) {

        var widgetMain = container.getWidget('WidgetAppointment');
        var widgetSwipe = container.getWidget('WidgetAppointmentSwipe');

        widgetSwipe.widgets[0].template = "widget/form-view.html";
        widgetSwipe.widgets[0].activate();
        widgetSwipe.widgets[0].view();

        widgetSwipe.widgets.forEach(function(widget) {
            widget.currentRow = widgetMain.currentRow;
        });

        Environment.setCurrentAppointmentDate(widgetMain.currentRow.date);

        ScreenService.openSwipe(widgetSwipe);

    };

    this.view = function(widget) {

        widget.template = "widget/form-view.html";

    };

    this.edit = function(widget) {

        widget.template = "widget/form.html";
        widget.edit();

    };

    this.add = function(widget) {

        widget.widgets[0].template = "widget/form.html";
        widget.widgets[0].edit();
        widget.widgets[0].newRow();

        widget.widgets[0].getField('date').value(Validation.now());

        ScreenService.openSwipe(widget);

    };

    this.editBio = function(widget) {

        var filterPatient = [{
                "name": "id",
                "operator": "=",
                "value": localStorageEngine.getLocalVar('__patient')
            },
            {
                "name": "unit",
                "operator": "=",
                "value": localStorageEngine.getLocalVar('__CDFILIAL')
            }
        ];

        localStorageEngine.getDataSource('/Patients', filterPatient).then(function() {
            widget.edit();
            widget.getField('examinationDate').readOnly = true;

        });

    };

    eventAggregator.subscribe('addBioAva', function(dataForNewAva) {

        var widget = dataForNewAva[0];
        var avaliationFields = widget.AvaliationFields;
        var dataSource = widget.localDataSourceLoaded;
        var patient = widget.curPatient;
        var currentAppointment = dataForNewAva[1];
        var now = new Date();
        var key = now.getTime();

        var newAvaliation = {
            'key': key,
            'age': patient.age.match(/[0-9]+/g),
            'gender': patient.gender,
            'appointmentDate': currentAppointment.date,
            'id': patient.id,
            'unit': patient.unit,
            '__is_new': true
        };

        if (patient.gender === '0') {
            newAvaliation.pregnancyId = 'N';
            avaliationFields.forEach(function(avfd) {
                if (avfd.name === "pregnancyWeek" || avfd.name === "pregnancyName") {
                    avfd.isVisible = false;
                }
            });
        } else {
            avaliationFields.forEach(function(avfd) {
                if (avfd.name === "pregnancyWeek" || avfd.name === "pregnancyName") {
                    avfd.isVisible = true;
                }
            });
        }

        var priority = dataSource.BioEvaluations.length + 1;
        newAvaliation.priority = Validation.strPad(priority, 3);
        // widget.widgets[0].getField('examinationName').readOnly = false;
        // widget.widgets[0].getField('examinationDate').readOnly = false;
        // console.log(newAvaliation);
        widget.edit();
        widget.avaliationExam = newAvaliation;
        widget.showAddEditExam = true;
    });

    this.cancelBio = function(widget) {
        var WidgetBiochemicalEvaluationGrid = widget.container.getWidget('WidgetBiochemicalEvaluationGrid');
        ScreenService.closeSwipe();
        WidgetBiochemicalEvaluationGrid.activate();
    };

    this.backBio = function(widget) {

        ScreenService.closeSwipe();
        widget.activate();

    };

    this.save = function(widget) {
        widget.currentRow.changed = true;
        if (widget.currentRow.dietName === "") {
            widget.currentRow.dietId = "";
            widget.currentRow.dietName = "sem descrição";
        }
        if (widget.currentRow.healthInsuranceName === "") {
            widget.currentRow.healthInsuranceId = "";
        }
        if (widget.currentRow.medicalTeamName === "") {
            widget.currentRow.medicalTeamId = "";
        }
        if (widget.currentRow.nutritionalStatusName === "") {
            widget.currentRow.nutritionalStatusId = "";
        }
        AppointmentService.save(widget.currentRow).then(function() {

            // ScreenService.showMessage('Salvo com sucesso.');
            if (widget.name === 'WidgetBiochemicalEvaluationGrid') {
                ScreenService.closeSwipe();
            } else {
                widget.template = "widget/form-view.html";
                ScreenService.closeSwipe();
                widget.container.getWidget('WidgetAppointment').reload();
                widget.container.getWidget('WidgetAppointment').activate();

            }
            Environment.lastPatientVisit();
        });

    };

    this.bioEvaluationSave = function(widget) {
        var WidgetBiochemicalEvaluationGrid = widget.container.getWidget('WidgetBiochemicalEvaluationGrid');
        var row = widget.currentRow;
        widget.currentRow.changed = true;
        if ((row.examinationDate) && (row.examinationName) && (row.examinationValue)) {
            AppointmentService.bioEvaluationSave(widget.currentRow).then(function() {

                // ScreenService.showMessage('Salvo com sucesso.');
                ScreenService.closeSwipe();
                WidgetBiochemicalEvaluationGrid.reload();
                WidgetBiochemicalEvaluationGrid.activate();
                Environment.lastPatientVisit();
            });
        } else {
            ScreenService.showMessage('Preencha os campos obrigatórios');
        }

    };

    function removeDuplicates(originalArray, objKey) {
        var trimmedArray = [];
        var values = [];
        var value;

        for (var i = 0; i < originalArray.length; i++) {
            value = originalArray[i][objKey];

            if (values.indexOf(value) === -1) {
                trimmedArray.push(originalArray[i]);
                values.push(value);
            }
        }

        return trimmedArray;

    }

    this.openSwipeAnthropometric = function(widget) {
        var widgetList = widget.container.getWidget('WidgetAnthropometricEvaluationList');
        var widgetTabbed = widget.container.getWidget('WidgetAnthropometricEvaluation');
        var filterPatient = [];
        if (widget.currentRow.__is_new) {
            var age = parseInt(widget.container.getWidget('WidgetNavigationStripe').currentRow.age.substring(0, 2));

            filterPatient = [{
                    name: "dataIni",
                    operator: "<",
                    value: age
                },
                {
                    name: "dataFim",
                    operator: ">",
                    value: age
                }
            ];

            localStorageEngine.getDataSource('/AnthEvaluationGeneral', filterPatient).then(function(methods) {

                methods = removeDuplicates(methods, 'name');

                widgetList.dataSource.data = methods;
                ScreenService.closePopup(true);
                ScreenService.openSwipe(widgetTabbed);
            });

        } else {
            filterPatient = [{
                    "name": "id",
                    "operator": "=",
                    "value": getLocalVar('__patient')
                },
                {
                    "name": "date",
                    "operator": "=",
                    "value": getLocalVar('__appointment')
                }
            ];

            localStorageEngine.getDataSource('/AnthEvaluationMethods', filterPatient).then(function(methods) {

                methods = removeDuplicates(methods, 'name');

                widgetList.dataSource.data = methods;
                ScreenService.closePopup(true);
                ScreenService.openSwipe(widgetTabbed);
            });

        }

    };

    this.openSwipeBiochemical = function(widget) {

        ScreenService.closePopup(true);
        ScreenService.openSwipe(widget.widgets[0]);

    };

    this.openSubWidgetSwipe = function(widget) {

        widget.widgets[0].widgets[0].activate();
        widget.widgets[0].widgets[0].currentRow = widget.currentRow;
        ScreenService.openSwipe(widget.widgets[0]);

    };

    this.classification = function(widget) {

        window.IDADE = widget.currentRow.age;
        window.RESULTADO = widget.currentRow.examinationValue;
        var formula = widget.currentRow.examinationFormula;

        if (widget.currentRow.gender === 0) {
            window.SEXO = MALE;
        } else {
            window.SEXO = FEMALE;
        }

        var status = Validation.formulaToJS(formula);

        widget.getField('examinationStatus').value(status);

    };

    this.anthDataEvaluation = function(widget) {

        widget.widgets[0].widgets[0].fields = [];
        widget.widgets[0].widgets[0].fieldGroups = "";
        AppointmentService.anthDataEvaluation(widget).then(function() {
            ScreenService.openSwipe(widget.widgets[0]);
        });

    };

    this.anthCalculate = function(widget) {
        AppointmentService.anthCalculate(widget).then(function() {
            ScreenService.openSwipe(widget.widgets[0]);
        });
    };

    this.widgetAfterSelectRow = function(checkedRows) {
        AppointmentService.saveCheckedRows(checkedRows);
    };

    this.backResult = function(widget) {

        ScreenService.closeSwipe();
        widget.container.getWidget('WidgetAnthropometricEvaluationSwipeForm').activate();

    };

    this.backEvaluation = function(widget) {

        ScreenService.closeSwipe();
        widget.container.getWidget('WidgetAnthropometricEvaluationList').activate();

    };

    this.backMethods = function(widget) {
        ScreenService.closeSwipe();
        widget.activate();
    };

    this.backOrCancel = function(args) {
        var deferred = ZHPromise.defer();
        if (args.row.__is_new) {
            args.owner.widget.list(true);
            args.owner.widget.dataSource.data.pop();
            args.owner.widget.currentRow = {};
            deferred.resolve();
        } else if (args.owner.widget.mode === 'edit') {
            args.owner.widget.view();
            for (var i in args.owner.widget.backupRow) {
                args.owner.widget.currentRow[i] = args.owner.widget.backupRow[i];
            }
            deferred.resolve();
            return deferred.promise;
        }
        ScreenService.goBack();
        return deferred.promise;
    };

    this.onlyNumbers = function(field) {
        if (isNaN(Number(field.value()))) {
            field.value(prevValue);
        } else {
            if (field.value() === "") {
                prevValue = "";
                field.value(0);
                field.value("");
                return false;
            }
            if (parseInt(Number(field.value())) === Number(field.value())) {
                field.value(parseInt(Number(field.value())).toString());
                field.value(Math.abs(field.value()));
                prevValue = field.value();
            } else {
                field.value(prevValue);
            }
        }
    };

    this.setPrevValue = function(field) {
        prevValue = field.value();
    };

};
Configuration(function(ContextRegister) {
    ContextRegister.register('AppointmentController', AppointmentController);
});
